// Copyright (C) 2022 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
// Qt-Security score:insignificant

#include "qqmlmodelsmodule_p.h"

QT_BEGIN_NAMESPACE

QT_END_NAMESPACE

#if QT_CONFIG(itemmodel)
#include "moc_qqmlmodelsmodule_p.cpp"
#endif
